#! /bin/sh
# /etc/init.d/dphys2-swapfile - automatically set up an swapfile
# author franklin, last modification 2003.12.04

# what we are
NAME=dphys2-swapfile

# this is what we want, 2 times RAM size
SWAPFACTOR=2
# this is a (outdated?) kernel limit (in MBytes), do not overrun it
MAXSWAP=2048

# let user overwride settings or size calculation
CONFFILE=/etc/default/${NAME}

# where we want the swapfile to be, this is the default
CONF_SWAPFILE=/var/swap


case "$1" in

  start|restart)

    echo -n "Starting ${NAME} swapfile setup"


    # compute automatic optimal size
    echo -n ", computing size"
    # this seems to be the nearest to physical RAM size, lacks about 60k
    KCORESIZE=`ls -l /proc/kcore | awk '{ print $5 }'`
    # make MBytes which rounded down will be exactly 1 too few, so add 1
    MEMSIZE=`expr ${KCORESIZE} / 1048576 + 1`
    # default, without config file overwriding, swap=2*RAM
    CONF_SWAPSIZE=`expr ${MEMSIZE} '*' ${SWAPFACTOR}`

    # check for user config overriding computed size
    if [ -f ${CONFFILE} ] ; then
      echo -n ", sourcing user config"
      # this can overwrite CONF_SWAPFILE or CONF_SWAPSIZE
      . ${CONFFILE}
    fi

    # announce end resulting config
    echo -n ": want ${CONF_SWAPFILE}=${CONF_SWAPSIZE}MByte"

    # check for legitimate swap size and restrict to it
    if expr ${CONF_SWAPSIZE} '>' ${MAXSWAP} > /dev/null ; then
      echo -n ", limiting to kernel limit: ${MAXSWAP}MBytes"
      CONF_SWAPSIZE=${MAXSWAP}
    fi


    # we will be later starting, and in between possible deleting/rebuilding
    #   so switch off any allready running swapfile, to avoid errors
    #   test here for running, as else also error from swapoff
    if [ x`swapon -s | grep ${CONF_SWAPFILE} | cut -f 1 -d " "` != x ] ; then
      swapoff ${CONF_SWAPFILE} 2>&1 > /dev/null
    fi


    # compare existing swapfile (if one exists) to see if it needs replacing
    if [ -f ${CONF_SWAPFILE} ] ; then

      echo -n ", checking existing"

      # we need bytes for comparing with existing swap file
      SWAPBYTES=`expr ${CONF_SWAPSIZE} '*' 1048576`

      FILEBYTES=`ls -l ${CONF_SWAPFILE} | awk '{ print $5 }'`

      # wrong size, get rid of existing swapfile, after remake
      if [ x${FILEBYTES} != x${SWAPBYTES} ] ; then

        # updates to this section need duplicating in postrm script
        #   can not simply make subroutine here and call that from postrm
        #     as this script is deleted before  postrm purge  is called

        echo -n ": deleting wrong size file (${FILEBYTES})"
        # remove any entry for swapfile from /etc/fstab
        #   do this first, so no deadlock if reboot and attempted swapon
        grep -v "^${CONF_SWAPFILE}" /etc/fstab > /etc/fstab.tmp
        # and get rid of comment about swapfile config
        grep -v "^# a swapfile" /etc/fstab.tmp > /etc/fstab
        # and the file is also gone ...
        rm ${CONF_SWAPFILE}

      else

        echo -n ": keeping it"

      fi
    fi

    # if no swapfile (possibly old one got deleted) make one
    if [ ! -f ${CONF_SWAPFILE} ] ; then

      echo -n ", generating swapfile ..."

      # first deleting existing lines, if any there, same as above
      grep -v "^${CONF_SWAPFILE}" /etc/fstab > /etc/fstab.tmp
      grep -v "^# swapfile" /etc/fstab.tmp > /etc/fstab

      dd if=/dev/zero of=${CONF_SWAPFILE} bs=1048576 \
        count=${CONF_SWAPSIZE} 2> /dev/null
      mkswap ${CONF_SWAPFILE} > /dev/null

      # do not mount swapfile via fstab, because S35mountall.sh is already done
      #   so just add warning comment line that swapfile is not in fstab
      echo -e "# a swapfile is not a swappartition, not swapon|off from here, \
use /etc/init.d/${NAME} for this" >> /etc/fstab

      # and inform user what we did
      echo -n " of ${CONF_SWAPSIZE}MBytes"

    fi

    # end out output line, else swapon will mess it up via syslog to console
    echo

    # as S35mountall.sh has already run, do this from here
    swapon ${CONF_SWAPFILE} 2>&1 > /dev/null

    echo "done."
    ;;


  stop)
    echo -n "Stopping ${NAME} swapfile setup"

    # as no entry in fstab, need to do this here,
    #   it is also symmetrical with swapon this way
    if [ x`swapon -s | grep ${CONF_SWAPFILE} | cut -f 1 -d " "` != x ] ; then
      swapoff ${CONF_SWAPFILE} 2>&1 > /dev/null
    fi

    echo ", done."
    ;;


  reload|force-reload)
    echo "No daemon to (force-)reload in ${NAME}"
    ;;


 *)
    echo "Usage: /etc/init.d/${NAME} {start|stop|restart}"

    exit 1
    ;;

esac

exit 0
